@@ -1,7 +1,7 @@
 /*
     ccid.h: CCID structures
     Copyright (C) 2003-2009   Ludovic Rousseau
-    Copyright (C) 2009-2013   Advanced Card Systems Ltd.
+    Copyright (C) 2009-2014   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -220,6 +220,7 @@ typedef struct
 #define KOBIL_TRIBANK	0x0D463010
 
 // CCID readers
+#define ACS_ACR32_ICC_READER			0x072fb301
 #define ACS_ACR33U_A1_3SAM_ICC_READER	0x072f8300
 #define ACS_ACR33U_A2_3SAM_ICC_READER	0x072f8302
 #define ACS_ACR33U_A3_3SAM_ICC_READER	0x072f8307
@@ -240,6 +241,10 @@ typedef struct
 #define ACS_ACR123_3S_READER			0x072f222e
 #define ACS_ACR1251_1S_CL_READER		0x072f221a
 #define ACS_ACR1251U_C					0x072f2218
+#define ACS_ACR1251K_DUAL_READER		0x072f2232
+#define ACS_ACR1251_1S_DUAL_READER		0x072f2242
+#define ACS_ACR1252_1S_CL_READER		0x072f223b
+#define ACS_ACR1261_1S_DUAL_READER		0x072f2211
 #define ACS_ACR128U						0x072f2100
 #define ACS_ACR1281_DUAL_READER_QPBOC	0x072f2208
 #define ACS_ACR1281_PICC_READER_BSI		0x072f0901
