@@ -1,7 +1,7 @@
 /*
     ccid_usb.c: USB access routines using the libusb library
     Copyright (C) 2003-2009   Ludovic Rousseau
-    Copyright (C) 2009-2013   Advanced Card Systems Ltd.
+    Copyright (C) 2009-2014   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -469,7 +469,9 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 							(ACS_ACR1281_DUAL_READER_BSI == readerID) ||
 							(ACS_ACR1281_1S_PICC_READER == readerID) ||
 							(ACS_ACR1251_1S_CL_READER == readerID) ||
-							(ACS_ACR1251U_C == readerID))
+							(ACS_ACR1251U_C == readerID) ||
+							(ACS_ACR1251K_DUAL_READER == readerID) ||
+							(ACS_ACR1252_1S_CL_READER == readerID))
 						{
 							// the CCID interfaces are 0 and 1
 							interface_number = static_interface - 1;
@@ -762,7 +764,9 @@ again:
 						usbDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
 						usbDevice[reader_index].ccid.bVoltageSupport = 0x03;
 					}
-					else if ((readerID == ACS_ACR1281_1S_DUAL_READER) ||
+					else if ((readerID == ACS_ACR1251_1S_DUAL_READER) ||
+						(readerID == ACS_ACR1261_1S_DUAL_READER) ||
+						(readerID == ACS_ACR1281_1S_DUAL_READER) ||
 						(readerID == ACS_ACR1281_2S_CL_READER))
 					{
 						usbDevice[reader_index].ccid.dwFeatures = 0x000204BA;
@@ -802,12 +806,20 @@ again:
 						// Simulate ACR85 as multi-slot reader
 						else if (readerID == ACS_ACR85_PINPAD_READER_ICC)
 							usbDevice[reader_index].ccid.bMaxSlotIndex = 1;
+						// Fix ACR32 incorrect max slot index
+						else if (readerID == ACS_ACR32_ICC_READER)
+							usbDevice[reader_index].ccid.bMaxSlotIndex = 0;
 						else
 							usbDevice[reader_index].ccid.bMaxSlotIndex = usb_interface->altsetting->extra[4];
 
 						usbDevice[reader_index].ccid.bCurrentSlotIndex = 0;
 						usbDevice[reader_index].ccid.readTimeout = DEFAULT_COM_READ_TIMEOUT;
-						usbDevice[reader_index].ccid.arrayOfSupportedDataRates = get_data_rates(reader_index, dev, num);
+
+						if (usb_interface->altsetting->extra[27] == 0)
+							usbDevice[reader_index].ccid.arrayOfSupportedDataRates = NULL;
+						else
+							usbDevice[reader_index].ccid.arrayOfSupportedDataRates = get_data_rates(reader_index, dev, num);
+
 						usbDevice[reader_index].ccid.bInterfaceProtocol = usb_interface->altsetting->bInterfaceProtocol;
 						usbDevice[reader_index].ccid.bNumEndpoints = usb_interface->altsetting->bNumEndpoints;
 						usbDevice[reader_index].ccid.dwSlotStatus = IFD_ICC_PRESENT;
@@ -856,7 +868,8 @@ again:
 					// The 2nd interface (composite device) is a SAM slot
 					if ((readerID == ACS_ACR1281_1S_PICC_READER) ||
 						(readerID == ACS_ACR1251_1S_CL_READER) ||
-						(readerID == ACS_ACR1251U_C))
+						(readerID == ACS_ACR1251U_C) ||
+						(readerID == ACS_ACR1252_1S_CL_READER))
 					{
 						if (interface == 1)
 							usbDevice[reader_index].ccid.isSamSlot = TRUE;
