@@ -1,7 +1,7 @@
 /*
     ifdhandler.c: IFDH API
     Copyright (C) 2003-2009   Ludovic Rousseau
-    Copyright (C) 2009-2013   Advanced Card Systems Ltd.
+    Copyright (C) 2009-2014   Advanced Card Systems Ltd.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -624,7 +624,9 @@ EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 						(ACS_ACR1281_DUAL_READER_BSI == readerID) ||
 						(ACS_ACR1281_1S_PICC_READER == readerID) ||
 						(ACS_ACR1251_1S_CL_READER == readerID) ||
-						(ACS_ACR1251U_C == readerID))
+						(ACS_ACR1251U_C == readerID) ||
+						(ACS_ACR1251K_DUAL_READER == readerID) ||
+						(ACS_ACR1252_1S_CL_READER == readerID))
 						*Value = 2;
 				}
 #endif
@@ -1063,7 +1065,7 @@ again:
 					// Cold reset
 					atrLen = sizeof(atrBuffer);
 					(void)IFDHPowerICC(Lun, IFD_POWER_DOWN, atrBuffer, &atrLen);
-					usleep(100 * 1000);
+					usleep(10 * 1000);
 					atrLen = sizeof(atrBuffer);
 					ret = IFDHPowerICC(Lun, IFD_POWER_UP, atrBuffer, &atrLen);
 					if (ret != IFD_SUCCESS)
@@ -1166,32 +1168,28 @@ again:
 
 		DEBUG_COMM2("Timeout: %d seconds", ccid_desc->readTimeout);
 
-		// Set parameters if not specific mode
-		if (!specificMode)
+		ret = ccid_slot->pSetParameters(reader_index, 1, sizeof(param), param);
+		if (IFD_SUCCESS != ret)
 		{
-			ret = ccid_slot->pSetParameters(reader_index, 1, sizeof(param), param);
-			if (IFD_SUCCESS != ret)
+			DEBUG_INFO("SetParameters (T1) Failed");
+
+			if (param[0] != 0x11)
 			{
-				DEBUG_INFO("SetParameters (T1) Failed");
-
-				if (param[0] != 0x11)
-				{
-					// Cold reset
-					atrLen = sizeof(atrBuffer);
-					(void)IFDHPowerICC(Lun, IFD_POWER_DOWN, atrBuffer, &atrLen);
-					usleep(100 * 1000);
-					atrLen = sizeof(atrBuffer);
-					ret = IFDHPowerICC(Lun, IFD_POWER_UP, atrBuffer, &atrLen);
-					if (ret != IFD_SUCCESS)
-						return ret;
-
-					// Try default Fl/Dl
-					pps[2] = 0x11;
-					goto again;
-				}
-				else
+				// Cold reset
+				atrLen = sizeof(atrBuffer);
+				(void)IFDHPowerICC(Lun, IFD_POWER_DOWN, atrBuffer, &atrLen);
+				usleep(10 * 1000);
+				atrLen = sizeof(atrBuffer);
+				ret = IFDHPowerICC(Lun, IFD_POWER_UP, atrBuffer, &atrLen);
+				if (ret != IFD_SUCCESS)
 					return ret;
+
+				// Try default Fl/Dl
+				pps[2] = 0x11;
+				goto again;
 			}
+			else
+				return ret;
 		}
 	}
 	else
@@ -1232,32 +1230,28 @@ again:
 		DEBUG_COMM2("Communication timeout: %d seconds",
 			ccid_desc->readTimeout);
 
-		// Set parameters if not specific mode
-		if (!specificMode)
+		ret = ccid_slot->pSetParameters(reader_index, 0, sizeof(param), param);
+		if (IFD_SUCCESS != ret)
 		{
-			ret = ccid_slot->pSetParameters(reader_index, 0, sizeof(param), param);
-			if (IFD_SUCCESS != ret)
+			DEBUG_INFO("SetParameters (T0) Failed");
+
+			if (param[0] != 0x11)
 			{
-				DEBUG_INFO("SetParameters (T0) Failed");
-
-				if (param[0] != 0x11)
-				{
-					// Cold reset
-					atrLen = sizeof(atrBuffer);
-					(void)IFDHPowerICC(Lun, IFD_POWER_DOWN, atrBuffer, &atrLen);
-					usleep(100 * 1000);
-					atrLen = sizeof(atrBuffer);
-					ret = IFDHPowerICC(Lun, IFD_POWER_UP, atrBuffer, &atrLen);
-					if (ret != IFD_SUCCESS)
-						return ret;
-
-					// Try default Fl/Dl
-					pps[2] = 0x11;
-					goto again;
-				}
-				else
+				// Cold reset
+				atrLen = sizeof(atrBuffer);
+				(void)IFDHPowerICC(Lun, IFD_POWER_DOWN, atrBuffer, &atrLen);
+				usleep(10 * 1000);
+				atrLen = sizeof(atrBuffer);
+				ret = IFDHPowerICC(Lun, IFD_POWER_UP, atrBuffer, &atrLen);
+				if (ret != IFD_SUCCESS)
 					return ret;
+
+				// Try default Fl/Dl
+				pps[2] = 0x11;
+				goto again;
 			}
+			else
+				return ret;
 		}
 	}
 
@@ -1396,6 +1390,14 @@ EXTERNAL RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 			return_value = CcidSlots[reader_index].pPowerOn(reader_index, &nlength, pcbuffer,
 				PowerOnVoltage);
 
+			// Fix ACR39U card power on problem.
+			if (return_value != IFD_SUCCESS)
+			{
+				nlength = sizeof(pcbuffer);
+				return_value = CcidSlots[reader_index].pPowerOn(reader_index, &nlength, pcbuffer,
+					PowerOnVoltage);
+			}
+
 			// Enable/Disable PICC
 			if (DriverOptions & DRIVER_OPTION_DISABLE_PICC)
 			{
@@ -1411,7 +1413,7 @@ EXTERNAL RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
 					while ((return_value != IFD_SUCCESS) || (nlength == 0))
 					{
 						(void)CcidSlots[reader_index].pPowerOff(reader_index);
-						usleep(100 * 1000);
+						usleep(10 * 1000);
 						nlength = sizeof(pcbuffer);
 						return_value = CcidSlots[reader_index].pPowerOn(reader_index, &nlength, pcbuffer,
 							PowerOnVoltage);
